/**
* This function returns the average Distance of all points of the contour to a virtual circle in the contour.
* Circles that doesnt fit the minPupilRadius or MaxPupilRadius criterium have a circularity of 1000, whats higher than every else value
**/
public static double circularity(Contour c){
  ArrayList <PVector> v=c.getPoints();
  Rectangle bound=c.getBoundingBox();
  double midx=midx(c);
  double midy=midy(c);
  double r=(bound.height+bound.width)/4;
  double d=0.0;
  for(int i=0;i<v.size();i++){
    PVector point=v.get(i);
    double dx=point.x-midx;
    double dy=point.y-midy;
    double dr=r-Math.sqrt(dx*dx+dy*dy);
    d+=Math.abs(dr);
    
  }
  d=d*10/v.size();
  if(r<minPupilRadius||r>maxPupilRadius) d=1000;
  return d;
}
/**
* This function recturns the middle x Value of the Box surrounding the contour
* @param c Contour
**/
public static double midx(Contour c){
  Rectangle bound=c.getBoundingBox();
  return bound.x+bound.width/2.0;
}

/**
* This function recturns the middle y Value of the Box surrounding the contour
* @param c Contour
**/
public static double midy(Contour c){
  Rectangle bound=c.getBoundingBox();
  return bound.y+bound.height/2.0;
}
/**
* This function draws Contours in Green Color
* @param contours ArrayList of Contours to draw
**/
void drawContours(ArrayList<Contour> contours){
  for (Contour contour : contours) {
    stroke(0, 255, 0);
    //contour.draw();
    
    stroke(255, 0, 0);
    beginShape();
    for (PVector point : contour.getPolygonApproximation().getPoints()) {
      vertex(point.x+480, point.y);
    }
    endShape();
  }
}
/**
* This function is not implemented yet, in future releases this Function should search brown colors and remove them, to allow people with brown eyes using this program
*
**/
PImage removeBrownIris(PImage eye){
  
  return null;
}
/**
* This Function will color the darkest pixel black all other pixel will be white.
* The number of dark pixels is determined with the pupilSize variable in Eyetracking tab
**/
PImage setThreshold(PImage img){
  img.loadPixels();
  int threshold=0;
  int blackpix=0;
  while(blackpix<pupilSize){
    blackpix=0;
    threshold++;
  for(int i=0;i<img.pixels.length;i++){
    if(brightness(img.pixels[i])<threshold){
      blackpix++;
    }
  }
  }
  /**threshold=100;
  for(int i=0;i<img.pixels.length;i++){
    if(Math.min(Math.min(red(img.pixels[i]),green(img.pixels[i])),blue(img.pixels[i]))<threshold) img.pixels[i]=color(0,0,0);
    else img.pixels[i]=color(255,255,255);
  }**/
  for(int i=0;i<img.pixels.length;i++){
    if(brightness(img.pixels[i])<threshold) img.pixels[i]=color(0,0,0);
    else img.pixels[i]=color(255,255,255);
  }
  img.updatePixels();
  return img;
}
/**
* The Function returns the contour with the lowest value of circularity. Returns the Index of the Contour with lowest circularity.
* @param c ArrayList of Contours
**/
int leastCircularity(ArrayList<Contour> c){
  int index=-1;
  double leastcircularity=10000;
    if(c!=null){
      for(int i=0;i<c.size();i++){
        double circularity=circularity(c.get(i));
        if(circularity<leastcircularity){
          index=i;
          leastcircularity=circularity;
        }
      }
    }
  return index;
}
