package Eyetracking3;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;

import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.MatOfDMatch;
import org.opencv.core.MatOfKeyPoint;
import org.opencv.core.Point;
import org.opencv.core.Scalar;
import org.opencv.features2d.BFMatcher;
import org.opencv.features2d.Features2d;
import org.opencv.features2d.ORB;
import org.opencv.highgui.HighGui;
import org.opencv.imgproc.Imgproc;

import processing.core.PApplet;
import processing.core.PImage;

public class ORBDetector {
	PApplet p;

	public ORBDetector(PApplet p) {
		this.p = p;

	}

	public MatOfDMatch match(Rectangle[] eyes2, BufferedImage image, Rectangle face, KeyPointDescriptor original,
			KeyPointDescriptor reference, boolean draw) {
		BFMatcher descriptorMatcher = BFMatcher.create(BFMatcher.BRUTEFORCE_HAMMING, true);
		MatOfDMatch matches = new MatOfDMatch();
		descriptorMatcher.match(reference.desc, original.desc, matches);
		Mat outImage = new Mat();
		Features2d.drawMatches(reference.getImage(), reference.getKeyPoint(), original.getImage(),
				original.getKeyPoint(), matches, outImage);
		BufferedImage img = (BufferedImage) HighGui.toBufferedImage(outImage);
		PImage desPoints = new PImage(img);
		desPoints.resize(480, 360);
		if (draw) {
			p.image(desPoints, 0, 0);

		}
		return matches;
	}

	public KeyPointsTrain trainDescriptor(Rectangle[] eyes2, BufferedImage image, Rectangle face, boolean draw) {
		OpenCV refcv;
		long l = 0;
		ORB b = ORB.create();

		image = image.getSubimage(160, 0, 960, 720);
		BufferedImage copyOfImage = new BufferedImage(image.getWidth(), image.getHeight(),
				BufferedImage.TYPE_3BYTE_BGR);
		Graphics g = copyOfImage.createGraphics();
		g.drawImage(image, 0, 0, null);

		Mat mat = new Mat(image.getHeight(), image.getWidth(), CvType.CV_8UC3);

		byte[] pixels = ((DataBufferByte) copyOfImage.getRaster().getDataBuffer()).getData();
		mat.put(0, 0, pixels);
		Imgproc.cvtColor(mat, mat, Imgproc.COLOR_BGR2GRAY);
		mat.convertTo(mat, CvType.CV_8UC1);
		Mat mask = new Mat(mat.rows(), mat.cols(), CvType.CV_8U, Scalar.all(0));
		// Imgproc.rectangle(mask, new Point(face.y*2,960-face.x*2), new
		// Point((face.y+face.height)*2,960-(face.x-face.width)*2), new
		// Scalar(255,255,255), -1);
		Imgproc.rectangle(mask, new Point(face.x * 2, face.y * 2),
				new Point((face.x + face.width) * 2, (face.y + face.height) * 2), new Scalar(255, 255, 255), -1);
		if (eyes2 != null) {
			for (int i = 0; i < eyes2.length; i++) {
				Imgproc.rectangle(mask, new Point(eyes2[i].x * 2, eyes2[i].y * 2),
						new Point((eyes2[i].x + eyes2[i].width) * 2, (eyes2[i].y + eyes2[i].height) * 2),
						new Scalar(0, 0, 0), -1);
			}
		}

		// mask.setTo( new Scalar( 0.0 ) );
		// mat.adjustROI(face.y, face.y+face.height, face.x, face.x+face.width);
		// System.out.println(face.width);
		// Rect r = ...;pt1
		MatOfKeyPoint point = new MatOfKeyPoint();

		Mat descriptor = new Mat();
		b.detect(mat, point, mask);
		b.compute(mat, point, descriptor);
		Mat out = new Mat();
		Features2d.drawKeypoints(mat, point, out);
		BufferedImage img = (BufferedImage) HighGui.toBufferedImage(out);
		PImage desPoints = new PImage(img);
		desPoints.resize(480, 360);
		if (draw) {
			p.image(desPoints, 0, 0);
		}
		return new KeyPointsTrain(point, descriptor, mat);

	}

	public KeyPointsReference query(Rectangle[] eyes2, BufferedImage image, Rectangle face, boolean draw,
			KeyPointDescriptor trainDescriptor) {
		KeyPointDescriptor desc = trainDescriptor(eyes2, image, face, draw);

		return new KeyPointsReference(desc.getKeyPoint(), desc.getDescriptors(), desc.getImage(),
				trainDescriptor.vectors);
	}
}
