package Eyetracking3;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.opencv.core.CvType;
import org.opencv.core.DMatch;
import org.opencv.core.KeyPoint;
import org.opencv.core.Mat;
import org.opencv.core.MatOfDMatch;
import org.opencv.core.MatOfKeyPoint;
import org.opencv.core.Point;
import org.opencv.core.Range;
import org.opencv.core.Scalar;
import org.opencv.features2d.Features2d;
import org.opencv.features2d.ORB;
import org.opencv.highgui.HighGui;
import org.opencv.imgproc.Imgproc;

import processing.core.PImage;

public class TrainImageThread implements Runnable {
	
	CamListener l;
	BufferedImage[] img;
	Rectangle[][] eyes;
	Rectangle[] face;
	int i;
	boolean collecting = false;
	KeyPointsTrain train;
	boolean descriptorReady=false;

	public TrainImageThread(int number, CamListener l) {
		this.l = l;
		img = new BufferedImage[number];
		eyes = new Rectangle[number][];
		face = new Rectangle[number];

	}

	@Override
	public void run() {
		collecting = true;
		for (int i = 0; i < img.length; i++) {
			img[i] = l.threadImage();
			eyes[i] = Eyetracking.eyes;
			face[i] = Eyetracking.lastface;

			System.out.println("Image");
			try {
				Thread.sleep(10000);

			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		collecting = false;
		
		train=trainDescriptor(eyes, img, face, false);
		descriptorReady=true;

	}

	public boolean isCollecting() {
		return collecting;
	}
	public KeyPointsTrain trainSet() {
		if(descriptorReady) 
			return train;
		else
			return null;
	}
	public KeyPointsTrain trainDescriptor(Rectangle[][] eyes2, BufferedImage[] image, Rectangle face[], boolean draw) {
		OpenCV refcv;
		long l = 0;
		ORB b = ORB.create();
		ArrayList<Mat> mat = new ArrayList<Mat>();
		ArrayList<Mat> mask = new ArrayList<Mat>();
		for (int i = 0; i < image.length; i++) {
			image[i] = image[i].getSubimage(160, 0, 960, 720);
			BufferedImage copyOfImage = new BufferedImage(image[i].getWidth(), image[i].getHeight(),
					BufferedImage.TYPE_3BYTE_BGR);
			Graphics g = copyOfImage.createGraphics();
			g.drawImage(image[i], 0, 0, null);

			mat.add(new Mat(image[i].getHeight(), image[i].getWidth(), CvType.CV_8UC3));

			byte[] pixels = ((DataBufferByte) copyOfImage.getRaster().getDataBuffer()).getData();
			mat.get(i).put(0, 0, pixels);
			Imgproc.cvtColor(mat.get(i), mat.get(i), Imgproc.COLOR_BGR2GRAY);
			mat.get(i).convertTo(mat.get(i), CvType.CV_8UC1);
			mask.add(new Mat(mat.get(i).rows(), mat.get(i).cols(), CvType.CV_8U, Scalar.all(0)));
			// Imgproc.rectangle(mask, new Point(face.y*2,960-face.x*2), new
			// Point((face.y+face.height)*2,960-(face.x-face.width)*2), new
			// Scalar(255,255,255), -1);
			Imgproc.rectangle(mask.get(i), new Point(face[i].x * 2, face[i].y * 2),
					new Point((face[i].x + face[i].width) * 2, (face[i].y + face[i].height) * 2),
					new Scalar(255, 255, 255), -1);
			if (eyes2 != null) {
				for (int index = 0; index < eyes2[i].length; index++) {
					Imgproc.rectangle(mask.get(i), new Point(eyes2[i][index].x * 2, eyes2[i][index].y * 2),
							new Point((eyes2[i][index].x + eyes2[i][index].width) * 2,
									(eyes2[i][index].y + eyes2[i][index].height) * 2),
							new Scalar(0, 0, 0), -1);
				}
			}

			// mask.setTo( new Scalar( 0.0 ) );
			// mat.adjustROI(face.y, face.y+face.height, face.x, face.x+face.width);
			// System.out.println(face.width);
			// Rect r = ...;pt1
		}
		ArrayList<MatOfKeyPoint> point = new ArrayList<MatOfKeyPoint>();

		ArrayList<Mat> descriptor = new ArrayList<Mat>();
		b.detect(mat, point, mask);
		b.compute(mat, point, descriptor);
		Mat out = new Mat();
		// Features2d.drawKeypoints(mat, point, out);
		// BufferedImage img = (BufferedImage) HighGui.toBufferedImage(out);
		// PImage desPoints = new PImage(img);
		// desPoints.resize(480, 360);
		ArrayList<Mat> rows = new ArrayList<Mat>();
		
		KeyPointsTrain t = new KeyPointsTrain(point.get(0), descriptor.get(0), mat.get(0));
		ArrayList<KeyPoint> allPoints=new ArrayList<KeyPoint>(t.getKeyPoint().toList());
		//System.out.println(t.getDescriptors().dump());
		
		System.out.println(t.getDescriptors().type());
		for (int tindex = 0; tindex < t.getDescriptors().rows(); tindex++) {
			rows.add(t.getDescriptors().row(tindex));
			System.out.println(rows.get(tindex).width());
		}
		for (int index = 1; index < mat.size(); index++) {
			ArrayList<Integer> remove = new ArrayList<Integer>();
			KeyPointsTrain t2 = new KeyPointsTrain(point.get(index), descriptor.get(index), mat.get(index));
			DMatch[] match = ORBDetector.staticmatch(image[index], face[index], t, t2, false).toArray();
			for (int matchindex = 0; matchindex < match.length; matchindex++) {
				remove.add(match[matchindex].queryIdx);
			}
			Collections.sort(remove, Collections.reverseOrder());
			ArrayList<KeyPoint> keypoints =new ArrayList<KeyPoint>( t2.pt.toList());
			
			Mat desc = t2.getDescriptors();
			ArrayList<Mat> rowsadd=new ArrayList<Mat>();
			for(int t2index=0;t2index<t2.getDescriptors().rows();t2index++) {
				rowsadd.add(desc.row(t2index));
				
			}
			for (int i = 0; i < remove.size(); i++) {
				rowsadd.remove((int) remove.get(i));
				keypoints.remove((int) remove.get(i));

			}
			rows.addAll(rowsadd);
			allPoints.addAll(keypoints);
			
			
		}
		Mat descriptorResult=new Mat(rows.size(),rows.get(0).width(),0);
		for(int i=0;i<rows.size();i++) {
			for(int j=0;j<descriptorResult.width();j++) {
				descriptorResult.put(i, j,rows.get(i).get(0, j));
			}
		}
		MatOfKeyPoint matKeyPointResult=new MatOfKeyPoint();
		matKeyPointResult.fromArray((KeyPoint[]) allPoints.toArray());
		return  new KeyPointsTrain(matKeyPointResult, descriptorResult, mat.get(i));

	}

}
