import java.awt.Rectangle;
import java.awt.geom.*;

public abstract class GameObject {
  private Rectangle hitbox; //hitbox of the game object

  
  public GameObject(Rectangle hitbox) {
    this.hitbox = hitbox;
  }
  
  //updates the position of the game object according to the speed of the level the object is part of.
  public void update(float speed) {
    moveTo(hitbox.getX()-speed, hitbox.getY());
  }
  
  //returns a value >0 if two game objects collide with each other. the returned value represents how big the intersection between both objects is
  public int collidesWith(GameObject other) {
      Area thisArea=new Area(getHitbox());
      Area otherArea=new Area(other.getHitbox());
      thisArea.intersect(otherArea);
      Rectangle r=thisArea.getBounds();
      return r.height;
  }
  
  //returns true if an object is in the visible frame. returns false otherwise.
  public boolean inSurface(){
    return hitbox.getX() <= width && hitbox.getX()+hitbox.getWidth() >= 0;
  }
  
  //prints the object filled with the passed color c 
  public void printHitbox(color c) {
    fill(c);
    rect((float)hitbox.getX(), (float)hitbox.getY(),(float)hitbox.getWidth(),(float)hitbox.getHeight());
  }
  
  //sets the objects hitbox to the passed hitbox.
  public void setHitbox(Rectangle hitbox) {
    this.hitbox = hitbox;
  }

  //returns the objects hitbox. 
  public Rectangle getHitbox() {
    return hitbox;
  }
  
  //moves the hitbox to the passed x and y values. x and y are the coordinates of the upper-left corner of the objects hitbox.
  public void moveTo(double x, double y){
    hitbox.setLocation((int)x,(int)y);
  }
}
