import java.util.Random;

public class Level {
  private Player player; //player in the Level.
  private final int startX = width/4; //start x-coordinate of the player.
  private final int startY = height-10; //start y-coordinate of the player and 
  private float gravitation; //gravitation of the Level.
  private float speed; //Speed of the Level.
  //private boolean win; //not relevant for the final version of our game
  
  private ArrayList<GameObject> objects; //List with all Objects and Triggers.
  private Segment lastSeg; //Segment generated last. 
 
  private long score; //Raw score.
  private final int scoreDiv=5; //Divisor to reduce the score to more resonable values. 
  

  public Level(Player player, float gravitation, float speed) {
    this.player = player;
    this.score = 0;
    this.gravitation = gravitation;
    this.speed = speed;
    player.moveTo(startX, startY-player.getHitbox().height);
    this.objects = new ArrayList<GameObject>();
    generateSegment();
  }
  
  //returns current score.
  public int getScore(){
    return (int) score/scoreDiv;
  } 
  
  //updates the position of all relevant game objects, increases the score, generates new and removes old obstacles,
  //detects collisions with obstacles and triggers 
  public boolean update() {
    if (player.getHealth().health!=0 /*&& !win*/) { //win is not relevant for the final version of our game
      score++;
      removeSegments();
      if(objects.size() == 10) generateSegment();
      player.update(gravitation, startY,speed);
      for (int i = 0; i < objects.size(); i++) {
        objects.get(i).update(speed);
        if(objects.get(i) instanceof Obstacle){
          int collision=objects.get(i).collidesWith(player);
          if (collision!=0) player.getHealth().changeHealth(-collision);  
        }
        else if(objects.get(i) instanceof Trigger){
          if (((Trigger)objects.get(i)).triggers()){
            //if (objects.get(i) instanceof Finish) win = true; //not relevant for the final version of our game
            if (objects.get(i) instanceof Gravity) invertGravitation();
          }
        }
      }
    return true;  
    }else return false;  
  }
  
  //prints all game objects in the visibile frame, the score and players health 
  public void print() {
    noStroke();
    background(255);
    player.printHitbox(color(0, 255, 0));
    for (int i = 0; i < objects.size(); i++) {
      if (objects.get(i).inSurface()){
        if (objects.get(i) instanceof Gravity) objects.get(i).printHitbox(color(255, 255, 0));
        if (objects.get(i) instanceof Obstacle) objects.get(i).printHitbox(color(50, 200, 255));
        //if (objects.get(i) instanceof Finish) objects.get(i).printHitbox(color(128)); //not relevant for the final version of our game
      }
    }
    fill(50,100,255);
    text("Score: "+score/scoreDiv,25,50);
    player.getHealth().print();

  } 
  
  //removes all obstacles which leave the visible frame. 
  private void removeSegments(){
    for(int i = 0; i<objects.size();i++){
      if(objects.get(i).getHitbox().getX() <-1*objects.get(i).getHitbox().getWidth()) objects.remove(i);
      else break;
    }  
  }
  
  //randomly selects a segment of obstacles positioned behind the last generated obstacle and adds it to the list of gameobjects 
  private void generateSegment(){
    Random rand = new Random();
    Segment nextSeg = null;
    int nextX = 0;
    boolean grav = false;
    int min = 0;
    if(lastSeg != null){
      nextX = (int)(objects.get(objects.size()-1).getHitbox().getX()+lastSeg.getAbstand());
      grav=lastSeg.getGrav();
    }else{
      nextX=startX+400;
      min=1;
    }
    do{
      switch(rand.nextInt(4)+min){
        case 0: 
          nextSeg = new Seg_Grav(nextX, grav); 
        break;
        case 1:
          nextSeg = new Seg_1(nextX, grav); 
        break;
        case 2:
          nextSeg = new Seg_2(nextX, grav); 
        break;
        case 3:
          nextSeg = new Seg_3(nextX, grav); 
        break;
        default: 
          nextSeg = new Seg_1(nextX, grav);
        break;
      }
    }while(lastSeg!= null && nextSeg.getClass() == lastSeg.getClass());  
    objects.addAll(nextSeg.getObjects());
    lastSeg=nextSeg;
  }  
  
  //inverts the gravitation of the level
  public void invertGravitation() {
    gravitation=-gravitation;
    player.negateJumpspeed();
  }
  
  //returns true, if the gravitations polarity is reversed. returns false otherwise.
  public boolean revGravity(){
    if(gravitation<0) return true;
    else return false;
  }
  
  //speeds up the levels speed by a certain factor and changes the gravitation, in such a way that the players jump height staies the same.
  public void speedUp(float factor){
    int oldJumpheight = player.getJumpheight(gravitation);
    speed*=factor;
    player.setJumpspeed(player.getJumpspeed()*factor); 
    
    while (Math.abs(oldJumpheight-player.getJumpheight(gravitation)) > 1){
      println(gravitation);
         if(gravitation<0) gravitation-=0.0001;
         else gravitation +=0.0001;
    }
  }
}
